﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class GameManager : MonoBehaviour {

	private static GameManager instance;

	public static GameManager Instance{get{return instance;}}

	public static Transform player;

	public GUIText scoreText;

	private float score;

	public float defaultScore;
	public string scoreNumberFormat, scorePrefix, scoreSuffix;

	public GUIText healthText;

	private float health;

	public float defaultHealth;

	public string healthNumberFormat, healthPrefix, healthSuffix;

	public bool enableDeathAtZeroHealth;

	//public Transform playerUnit;

	private bool paused, forcePaused;

	public Transform playerPrefab;
	public Transform playerSpawn;

	public Transform enemyPrefab;
	public Transform[] enemySpawns;
	public Transform enemyParent, bulletParent, tombstoneParent;

	public float initialSpawnTime, spawnDivider, minSpawnTimer;
	private float nextSpawnTime, nextSpawn;

	public GUIText pressAnyKeyText;

	public Transform deathScreen;
	public float deathScreenTimer;
	private bool dead;
	private float deathScreenStartTime;
	public Transform introScreen;
	public float introScreenTimer;
	private bool onIntro;
	private float introScreenStartTime;
	private Transform activeCutsceneScreen;

	public Transform playerDieSound;
	public Transform musicObject;
	private AudioSource gameMusic;
	// Use this for initialization
	void Start () {
		if(instance == null){
			instance = this;
			//player = playerUnit;
		}
		gameMusic = musicObject.GetComponent<AudioSource>();
		if (gameMusic == null) print (this.name + " expected an AudioSource on " + musicObject.name + " but couldn't find one!");

		intro();
	}
	
	// Update is called once per frame
	void Update () {
		if(Input.GetButtonDown ("PauseGame") && !forcePaused){ //let's let people pause it
			pause();
			//print ("presspause");
		}

		if(Input.GetButtonDown ("QuitGame")){
			Application.Quit();
		}

		if(dead && Time.time > deathScreenStartTime + deathScreenTimer){
			if(Input.anyKeyDown) resetGame ();
			pressAnyKeyText.enabled = true;
		}

		if(onIntro && Time.time > introScreenStartTime + introScreenTimer){
			if(Input.anyKeyDown) resetGame ();
			pressAnyKeyText.enabled = true;
		}

		if(!dead && !onIntro) pressAnyKeyText.enabled = false;

		if(Time.time > nextSpawn && !dead && !onIntro) spawnEnemy();
	}

	public static float getScore(){
		return Instance.score;
	}

	public static void changeScore(float change){
		Instance.score += change;
		Instance.scoreText.text = Instance.scorePrefix + getScore().ToString(Instance.scoreNumberFormat) + Instance.scoreSuffix;
	}

	public static float getHealth(){
		return Instance.health;
	}

	public static void changeHealth(float change){
		Instance.health += change;
		Instance.healthText.text = Instance.healthPrefix + getHealth().ToString(Instance.healthNumberFormat) + Instance.healthSuffix;
		if(Instance.enableDeathAtZeroHealth && getHealth() == 0) Instance.die();
	}

	public void resetGame(){
		dead = false;
		onIntro = false;
		scoreText.enabled = true;
		score = defaultScore;
		changeScore (0);
		healthText.enabled = true;
		health = defaultHealth;
		changeHealth(0);
		paused = false;
		forcePaused = false;
		pressAnyKeyText.enabled = false;
		if(activeCutsceneScreen != null) Destroy(activeCutsceneScreen.gameObject);
		if(!gameMusic.isPlaying) gameMusic.Play();

		List<GameObject> bulletChildren = new List<GameObject>();
		foreach (Transform child in bulletParent){
			bulletChildren.Add(child.gameObject);
		}
		foreach (GameObject child in bulletChildren){
			Destroy(child);
		}

		List<GameObject> enemyChildren = new List<GameObject>();
		foreach (Transform child in enemyParent){
			enemyChildren.Add(child.gameObject);
		}
		foreach (GameObject child in enemyChildren){
			Destroy(child);
		}

		List<GameObject> tombstoneChildren = new List<GameObject>();
		foreach (Transform child in tombstoneParent){
			tombstoneChildren.Add(child.gameObject);
		}
		foreach (GameObject child in tombstoneChildren){
			Destroy(child);
		}


		if(player != null) Destroy(player.gameObject);
		player = (Transform) Instantiate (playerPrefab, playerSpawn.position, playerSpawn.rotation);

		spawnEnemy();
		nextSpawn = Time.time + initialSpawnTime;
		nextSpawnTime = initialSpawnTime;
	}

	public static bool isPaused(){
		return Instance.paused || Instance.dead || Instance.onIntro;
	}

	private void spawnEnemy(){
		Transform spawnPoint = enemySpawns[Random.Range(0, enemySpawns.Length)];
		Transform newEnemy = (Transform) Instantiate(enemyPrefab, spawnPoint.position, spawnPoint.rotation);
		BadGuyScript bgs = newEnemy.GetComponent<BadGuyScript>();
		if(bgs != null){ // he's supposed to be a baddy, but whatever
			bgs.target = player;
		}
		if(nextSpawnTime > minSpawnTimer){
			nextSpawnTime /= spawnDivider;
			nextSpawn = Time.time + nextSpawnTime;
		} else {
			nextSpawn = Time.time + minSpawnTimer;
		}
		//print(nextSpawnTime);
		newEnemy.parent = enemyParent;
	}

	private void pause(){
		if(!forcePaused){
			if(paused){
				Time.timeScale = 1;
				paused = false;
			} else {
				Time.timeScale = 0;
				paused = true;
			}
		}
	}

	private void forcePause(){
		forcePaused = true;
		paused = true;
		Time.timeScale = 0;
	}

	private void forceUnpause(){
		forcePaused = false;
		paused = false;
		Time.timeScale = 0;
	}

	private void die(){
		//if(!paused) pause ();
		dead = true;
		pressAnyKeyText.enabled = false;
		deathScreenStartTime = Time.time;
		activeCutsceneScreen = (Transform) Instantiate(deathScreen);
		scoreText.enabled = false;
		healthText.enabled = false;
		gameMusic.Stop();
		if(playerDieSound != null) Instantiate(playerDieSound, player.position, player.rotation);
	}

	private void intro(){
		onIntro = true;
		pressAnyKeyText.enabled = false;
		player = (Transform) Instantiate (playerPrefab, playerSpawn.position, playerSpawn.rotation);
		introScreenStartTime = Time.time;
		activeCutsceneScreen = (Transform) Instantiate(introScreen);
		scoreText.enabled = false;
		healthText.enabled = false;
	}
}
